/**
 * @function doCreateSignPass
 */
'use strict'

const co = require('co')
const errors = require('../errors')
const successPayload = require('apeman-app-rest/lib/payloads/success_payload')
const failurePayload = require('apeman-app-rest/lib/payloads/failure_payload')
const validateDataType = require('apeman-app-rest/lib/validating/validate_data_type')
const apemanmodel = require('apemanmodel')
const uuid = require('uuid')
const NameConstants = require('../constants/name_constants')

/** @lends doCreateSignPass */
function doCreateSignPass (config) {
  const ACTION_NAME = 'create'
  let success = successPayload(ACTION_NAME)
  let failure = failurePayload(ACTION_NAME)

  let digest = config.digest
  let name = NameConstants.SIGN_PASS_NAME

  function createSignPass (UserSign, context) {
    let transaction = context.transaction
    let data = context.data
    let key = data.attributes.key
    let id = uuid.v4()
    return co(function * () {
      let sign = yield UserSign.findOne({
        transaction,
        where: { key },
        include: [
          UserSign.associations[ 'user' ]
        ]
      })
      if (!sign) {
        return failure(errors.signError())
      }

      let typeError = validateDataType(data, apemanmodel.toType(name))
      if (typeError) {
        return failure(typeError)
      }

      let hash = yield digest(data.attributes.password, sign.hashSalt)
      if (!hash === sign.hash) {
        return failure(errors.signError())
      }

      let req = context.req
      req.signed({
        id: id,
        sign: sign
      })
      return success(
        {},
        {
          type: apemanmodel.toType(name),
          id: id,
          relationships: {
            sign: {
              data: apemanmodel.toData.withoutAttributes(UserSign, sign)
            }
          }
        },
        {},
        [
          apemanmodel.toData(UserSign, sign)
        ]
      )
    })
  }

  return createSignPass
}

module.exports = doCreateSignPass
