/**
 * @function doDestroySignPass
 */
'use strict'

const co = require('co')
const errors = require('../errors')
const successPayload = require('apeman-app-rest/lib/payloads/success_payload')
const failurePayload = require('apeman-app-rest/lib/payloads/failure_payload')
const NameConstants = require('../constants/name_constants')

/** @lends doDestroySignPass */
function doDestroySignPass (config) {
  const ACTION_NAME = 'destroy'
  let success = successPayload(ACTION_NAME)
  let fail = failurePayload(ACTION_NAME)
  let name = NameConstants.SIGN_PASS_NAME

  function destroySignPass (UserSign, context) {
    let id = context.id
    return co(function * () {
      let req = context.req
      let signed = req.signed()
      if (!signed) {
        return success({ count: 0 })
      }
      if (signed.id !== id) {
        return fail(errors.notFoundError(name, id))
      }
      req.signed({})
      return success({ count: 1 })
    })
  }

  Object.assign(destroySignPass, {
    bulk (UserSign, context) {
      return co(function * () {
        let req = context.req
        let signed = req.signed()
        if (!signed) {
          return success({ count: 0 })
        }
        req.signed(null)
        return success({ count: 1 })
      })
    }
  })

  return destroySignPass
}

module.exports = doDestroySignPass

