/**
 * @function doListSignPass
 */
'use strict'

const co = require('co')
const apemanmodel = require('apemanmodel')
const successPayload = require('apeman-app-rest/lib/payloads/success_payload')
const NameConstants = require('../constants/name_constants')

/** @lends doListSignPass */
function doListSignPass (config) {
  const ACTION_NAME = 'list'
  let success = successPayload(ACTION_NAME)
  let name = NameConstants.SIGN_PASS_NAME

  let _includedAfterList = (included) => co(function * () {
    for (let i of [].concat(included || [])) {
      let attributes = i.attributes
      delete attributes.hash
      delete attributes.hashSalt
    }
    return included
  })

  function listSignPass (UserSign, context) {
    return co(function * () {
      let req = context.req
      let signed = req.signed()
      if (!signed) {
        return success({}, [])
      }
      let sign = apemanmodel.dynamicModel(signed.sign)
      let result = success(
        {},
        [
          {
            type: apemanmodel.toType(name),
            relationships: {
              sign: {
                data: apemanmodel.toData.withoutAttributes(UserSign, sign)
              }
            }
          }
        ],
        {},
        [
          apemanmodel.toData(UserSign, sign, {})
        ]
      )
      result.included = yield _includedAfterList(result.included)
      return result
    })
  }

  return listSignPass
}

module.exports = doListSignPass
